﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Browser;
using System.Xml.Linq;
using System.Linq;
using System.Collections.Generic;

namespace MashupImageSearch.Model
{
    public class DaumPhotoService : PhotoServiceBase
    {
        /// <summary>
        /// 다음 오픈 API 키 (여기에서 발급 : https://apis.daum.net/register/apikey.daum)
        /// </summary>
        private readonly string API_KEY = "df6da85e924b33bfdb71044ce40569e1e97557a2";

        /// <summary>
        /// 요청 URL 포맷
        /// q : 질의어 (키워드)
        /// result : 검색 결과수 (최대 20개 x_x)
        /// pageno : 검색 페이지 (최대 500페이지)
        /// sort : 0 - 최신순, 1 - 정확도순
        /// output : 결과형식 (json, xml)
        /// </summary>
        private readonly string REQUEST_URL_FORMAT = "http://apis.daum.net/search/image?apikey={0}&q={1}&result={2}&pageno={3}&sort=0&output=xml";

        #region IPhotoService Members

        public override void SearchImageAsync(string keyword, int page, int perPage)
        {
            // 다음은 최대 검색수가 20이기때문
            perPage = Math.Min(perPage, 20);

            string requestUrl = string.Format(REQUEST_URL_FORMAT, API_KEY, HttpUtility.UrlEncode(keyword), perPage, page);

            WebClient wc = new WebClient();
            wc.DownloadStringCompleted += new DownloadStringCompletedEventHandler(wc_DownloadStringCompleted);
            wc.DownloadStringAsync(new Uri(requestUrl, UriKind.Absolute), keyword);
        }

        void wc_DownloadStringCompleted(object sender, DownloadStringCompletedEventArgs e)
        {
            if (e.Cancelled == true ||
                e.Error != null ||
                string.IsNullOrEmpty(e.Result) == true)
                return;

            XDocument xDoc = XDocument.Parse(e.Result);
            if (xDoc == null)
                return;

            var query = from c in xDoc.Descendants("item")
                        select new Photo
                        {
                            Title = c.Element("title").Value,
                            Link = c.Element("link").Value,
                            Url = c.Element("image").Value,
                            ThumbnailUrl = c.Element("thumbnail").Value
                        };

            List<Photo> photos = new List<Photo>();
            foreach (Photo photo in query)
            {
                // 확장자의 위치
                int posExt = photo.ThumbnailUrl.LastIndexOf(".");
                if (posExt == -1) // 확장자가 없으면
                    continue;

                string ext = photo.ThumbnailUrl.Substring(posExt + 1);
                if (ext.ToLower().CompareTo("gif") == 0) // 확장자가 gif이면
                    continue;

                photos.Add(photo);
            }

            OnSearchImageCompleted(e.UserState.ToString(), photos);
        }

        #endregion
    }
}
