﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections.Generic;
using System.Xml.Linq;
using System.Linq;
using System.Windows.Browser;

namespace MashupImageSearch.Model
{
    /// <summary>
    /// 플리커로부터 사진을 검색해주는 서비스
    /// </summary>
    public class FlickrPhotoService : PhotoServiceBase
    {
        /// <summary>
        /// 어플리케이션 키 (여기에서 발급 : http://www.flickr.com/services/api/keys/)
        /// </summary>
        private readonly string API_KEY = "10508b206a3cc706aa7efb90c3776ffb";

        private readonly string THUMBNAIL_IMAGEURL_FORMAT = "http://farm{0}.static.flickr.com/{1}/{2}_{3}_t.jpg";
        private readonly string ORIGINAL_IMAGEURL_FORMAT = "http://farm{0}.static.flickr.com/{1}/{2}_{3}_m.jpg";
        private readonly string PHOTO_PAGEURL_FORMAT = "http://www.flickr.com/photos/{0}/{1}";

        private readonly string SEARCH_FORMAT = "http://api.flickr.com/services/rest/?method=flickr.photos.search&api_key={0}&text={1}&per_page={2}&page={3}";

        private enum PhotoTypes
        {
            Original = 0,
            Thumbnail = 1
        }

        #region Methods


        /// <summary>
        /// Thumbnail 이미지의 주소를 생성하여 반환합니다.
        /// </summary>
        /// <param name="farm"></param>
        /// <param name="server"></param>
        /// <param name="id"></param>
        /// <param name="secret"></param>
        /// <returns></returns>
        private string GetThumbnailImageUrl(string farm, string server, string id, string secret)
        {
            return string.Format(THUMBNAIL_IMAGEURL_FORMAT, farm, server, id, secret);
        }

        /// <summary>
        /// Original 이미지의 주소를 생성하여 반환합니다.
        /// </summary>
        /// <param name="farm"></param>
        /// <param name="server"></param>
        /// <param name="id"></param>
        /// <param name="secret"></param>
        /// <returns></returns>
        private string GetOriginalImageUrl(string farm, string server, string id, string secret)
        {
            return string.Format(ORIGINAL_IMAGEURL_FORMAT, farm, server, id, secret);
        }

        /// <summary>
        /// 사진이 포함된 페이지의 주소
        /// </summary>
        /// <param name="owner"></param>
        /// <param name="photoId"></param>
        /// <returns></returns>
        private string GetPhotoPageUrl(string owner, string photoId)
        {
            return string.Format(PHOTO_PAGEURL_FORMAT, owner, photoId);
        }

        #endregion Methods


        #region IPhotoService Members

        public override void SearchImageAsync(string keyword, int page, int perPage)
        {
            if (string.IsNullOrEmpty(keyword) == true)
                return;

            string requestUrl = string.Format(SEARCH_FORMAT, API_KEY, HttpUtility.UrlEncode(keyword), perPage, page);

            WebClient wc = new WebClient();
            wc.DownloadStringCompleted += new DownloadStringCompletedEventHandler(wc_DownloadStringCompleted);
            wc.DownloadStringAsync(new Uri(requestUrl, UriKind.Absolute), keyword);
        }

        void wc_DownloadStringCompleted(object sender, DownloadStringCompletedEventArgs e)
        {
            if (e.Cancelled == true ||
                e.Error != null ||
                string.IsNullOrEmpty(e.Result) == true)
                return;

            XDocument xDoc = XDocument.Parse(e.Result);
            if (xDoc == null)
                return;

            var query = from c in xDoc.Descendants("photo")
                        select new
                        {
                            Farm = c.Attribute("farm").Value,
                            Server = c.Attribute("server").Value,
                            ID = c.Attribute("id").Value,
                            Secret = c.Attribute("secret").Value,
                            Title = c.Attribute("title").Value,
                            Owner = c.Attribute("owner").Value
                        };

            List<Photo> photos = new List<Photo>();
            foreach (var item in query)
            {
                photos.Add(new Photo
                {
                    Url = GetOriginalImageUrl(item.Farm, item.Server, item.ID, item.Secret),
                    ThumbnailUrl = GetThumbnailImageUrl(item.Farm, item.Server, item.ID, item.Secret),
                    Title = item.Title,
                    Link = GetPhotoPageUrl(item.Owner, item.ID)
                });
            }

            OnSearchImageCompleted(e.Result.ToString(), photos);
        }

        #endregion
    }
}
